package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.system.entity.*;
import pps.core.system.error.LoginError;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_user.*;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@XText("用户管理")
@XService
public class SysUserService {

    @XText("查询用户列表")
    @XApiPost
    public XPageResult<QuerySysUserViewOutput> selectSysUserViewPage(XContext context, QuerySysUserViewInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        SysUserView record = new SysUserView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserView> list;
        record.setEndTime(new Date());
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (BusinessConstant.ZERO.equals(session.getIsSuperAdmin())) {
            if (CharSequenceUtil.isBlank(input.getOuId())) {
                record.setOuId(session.getOuId());
            }
        }
        if (ObjectUtil.isNotNull(input.getHasOuId()) && input.getHasOuId().equals(0)) {
            record.setHasOuId(input.getHasOuId().toString());
            list = mapper.selectSysUserListByNoOuId(record);
        } else
            list = mapper.selectSysUserListByOuId(record);
        PageInfo<SysUserView> pageInfo = new PageInfo<>(list);
        List<QuerySysUserViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserViewOutput.class);
        for (QuerySysUserViewOutput output : outputs) {
            output.setPhone(UserInfoUtil.desensitization(output.getPhone()));
            output.setIdentifyNo(UserInfoUtil.desensitization(output.getIdentifyNo()));
            output.setEmail(UserInfoUtil.desensitization(output.getEmail()));
            if (ObjectUtil.isNull(output.getStatus())) {
                output.setStatus(0);
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("新增用户")
    @XApiPost
    public XServiceResult createSysUser(XContext context, CreateSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            //查询是否存在相同的登录名，如果存在，提示错误信息
            List<SysUserEnt> isExit = mapper.selectList(new LambdaQueryWrapper<SysUserEnt>()
                    .eq(SysUserEnt::getLoginName, input.getLoginName()));
            if (CollUtil.isNotEmpty(isExit)) {
                return XServiceResult.error(context, LoginError.Account_Already_Exists);
            }
            //查询是否存在有相同的Iam登录名，如果存在，提示错误信息
            if (CharSequenceUtil.isNotBlank(input.getIamLoginName())) {
                List<SysUserEnt> isIamExit = mapper.selectList(new LambdaQueryWrapper<SysUserEnt>()
                        .eq(SysUserEnt::getIamLoginName, input.getIamLoginName()));
                if (CollUtil.isNotEmpty(isIamExit)) {
                    return XServiceResult.error(context, LoginError.Already_Exists_IAM);
                }
            }
            SysUserEnt entity = new SysUserEnt();
            XCopyUtils.copyObject(input, entity);
            if (CharSequenceUtil.isNotBlank(input.getPassword())) {
                return XServiceResult.error(context, LoginError.Missing_Pd);
            }
            entity.setPassword(input.getPassword());
            String uuid = BaseUtils.randomUUIDString();
            entity.setId(uuid);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (ObjectUtil.isNotNull(session)) {
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
            }
            entity.setCreateTime(new Date());
            //加密敏感信息
            entity.setPhone(UserInfoUtil.encrypt(entity.getPhone()));
            entity.setIdentifyNo(UserInfoUtil.encrypt(entity.getIdentifyNo()));
            entity.setEmail(UserInfoUtil.encrypt(entity.getEmail()));
            mapper.insert(entity);
            //新增用户和组织机构关系表
            insertSysUserOrganizationRel(context, uuid, input.getOuId(), session);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    //新增用户和组织机构关系表
    public void insertSysUserOrganizationRel(XContext context, String uuid, String ouId, PpsUserSession session) throws XServiceException {
        XTransactionHelper.begin(context, () -> {
            SysUserOrganizationRelMapper sorMapper = context.getBean(SysUserOrganizationRelMapper.class);
            SysUserOrganizationRelEnt relEnt = new SysUserOrganizationRelEnt();
            relEnt.setId(BaseUtils.randomUUIDString());
            relEnt.setUserId(uuid);
            relEnt.setOuId(ouId);
            Date date = new Date();
            relEnt.setEffectTime(date);
            relEnt.setEndTime(DateUtil.parseDate("9999-12-31"));
            if (ObjectUtil.isNotNull(session)) {
                relEnt.setCreateById(session.getId());
                relEnt.setCreateByName(session.getUserName());
            }
            relEnt.setCreateTime(date);
            sorMapper.insert(relEnt);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("修改用户信息")
    @XApiPost
    public XServiceResult updateSysUser(XContext context, UpdateSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysUserEnt::getId, input.getId());
            SysUserEnt entity = mapper.selectOne(queryWrapper);
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //查询是否存在相同的登录名，如果存在，提示错误信息
            queryWrapper.clear();
            queryWrapper.lambda().eq(SysUserEnt::getLoginName, input.getLoginName());
            List<SysUserEnt> isExit = mapper.selectList(queryWrapper);
            if (CollUtil.isNotEmpty(isExit) && (isExit.size() > 1 || !CharSequenceUtil.equals(isExit.get(0).getId(), input.getId()))) {
                return XServiceResult.error(993, "登录账户已存在！");
            }
            //查询是否存在有相同的Iam登录名，如果存在，提示错误信息
            if (CharSequenceUtil.isNotBlank(input.getIamLoginName())) {
                queryWrapper.clear();
                queryWrapper.lambda().eq(SysUserEnt::getIamLoginName, input.getIamLoginName());
                List<SysUserEnt> isIamExit = mapper.selectList(queryWrapper);
                if (CollUtil.isNotEmpty(isIamExit) && (isIamExit.size() > 1 || !CharSequenceUtil.equals(isIamExit.get(0).getId(), input.getId()))) {
                    return XServiceResult.error(993, "Iam登录账户已存在！");
                }
            }
            XCopyUtils.copyObject(input, entity);
            if (CharSequenceUtil.isNotBlank(entity.getPassword())) {
                entity.setPassword(input.getPassword());
            }
            entity.setModifyTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (ObjectUtil.isNotNull(session)) {
                entity.setModifyById(session.getId());
                entity.setModifyByName(session.getUserName());
            }
            //加密敏感信息

            if (CharSequenceUtil.isNotBlank(input.getPhone()) && !CharSequenceUtil.contains(input.getPhone(), '*')) {
                entity.setPhone(UserInfoUtil.encrypt(input.getPhone()));
            } else {
                entity.setPhone(null);
            }
            if (CharSequenceUtil.isNotBlank(input.getIdentifyNo()) && !CharSequenceUtil.contains(input.getIdentifyNo(), '*')) {
                entity.setIdentifyNo(UserInfoUtil.encrypt(input.getIdentifyNo()));
            } else {
                entity.setIdentifyNo(null);
            }
            if (CharSequenceUtil.isNotBlank(input.getEmail()) && !CharSequenceUtil.contains(input.getEmail(), '*')) {
                entity.setEmail(UserInfoUtil.encrypt(input.getEmail()));
            } else {
                entity.setEmail(null);
            }
            mapper.updateById(entity);
            //修改用户和组织机构关系为失效
            updateSysUserOrganizationRel(context, input.getId(), input.getOuId(), session, "update");
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("修改用户状态（封禁、解锁）")
    @XApiPost
    public XServiceResult updateSysUserStatus(XContext context, UpdateSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            SysUserEnt entity = mapper.selectOne(new QueryWrapper<SysUserEnt>()
                    .lambda()
                    .eq(SysUserEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            entity.setStatus(input.getStatus());
            entity.setModifyTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            mapper.updateById(entity);
            if (ObjectUtil.isNotNull(input.getStatus()) && input.getStatus().equals(0)) {
                //删除登录日志错误信息
                SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
                userLoginLogMapper.delete(new QueryWrapper<SysUserLoginLogEnt>()
                        .lambda()
                        .eq(SysUserLoginLogEnt::getLoginName, entity.getLoginName()));
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    public void updateSysUserOrganizationRel(XContext context, String userId, String ouId, PpsUserSession session, String flag) throws XServiceException {
        XTransactionHelper.begin(context, () -> {
            //修改用户和组织机构关系为失效
            SysUserOrganizationRelMapper relMapper = context.getBean(SysUserOrganizationRelMapper.class);
            Date date = new Date();
            SysUserOrganizationRelEnt relEntity = relMapper.selectOne(new QueryWrapper<SysUserOrganizationRelEnt>()
                    .lambda()
                    .eq(SysUserOrganizationRelEnt::getUserId, userId)
                    .le(SysUserOrganizationRelEnt::getEffectTime, date)
                    .ge(SysUserOrganizationRelEnt::getEndTime, date));
            if (ObjectUtil.isNull(relEntity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //如果是修改就新增一条，删除的话不需要新增
            if (CharSequenceUtil.equals("update", flag)) {
                //如果不相等改为失效
                if (!CharSequenceUtil.equals(relEntity.getOuId(), ouId)) {
                    relEntity.setEndTime(date);
                    relEntity.setModifyById(session.getId());
                    relEntity.setModifyByName(session.getUserName());
                    relEntity.setModifyTime(date);
                    relMapper.updateById(relEntity);
                    //新增一条记录
                    insertSysUserOrganizationRel(context, userId, ouId, session);
                }
            } else {
                relEntity.setEndTime(date);
                relEntity.setModifyById(session.getId());
                relEntity.setModifyByName(session.getUserName());
                relEntity.setModifyTime(date);
                relMapper.updateById(relEntity);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("删除用户")
    @XApiPost
    public XServiceResult deleteSysUser(XContext context, DeleteSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            SysUserEnt entity = mapper.selectOne(new QueryWrapper<SysUserEnt>()
                    .lambda()
                    .eq(SysUserEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            mapper.deleteById(entity);
            //修改用户和组织机构表为失效
            updateSysUserOrganizationRel(context, input.getId(), null, context.getSession(PpsUserSession.class), "delete");
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("根据组织机构id分页获取用户列表")
    @XApiPost
    public XPageResult<QuerySysUserViewOutput> getUserListByOuIdPage(XContext context, GetUserListByOuIdInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        SysUserView record = new SysUserView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        if (CharSequenceUtil.isBlank(record.getOuId())) {
            SysOrganizationViewMapper organizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
            SysOrganizationView sysOrganizationView = new SysOrganizationView();
            sysOrganizationView.setEndTime(new Date());
            sysOrganizationView.setIsDeleted(BusinessConstant.ONE);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            List<SysOrganizationView> viewList;
            if (ObjectUtil.isNull(input.getShowAll()) || !input.getShowAll()) {
                sysOrganizationView.setId(session.getOuId());
                viewList = organizationViewMapper.selectOuByOuId(sysOrganizationView);
                if (CollUtil.isEmpty(viewList)) {
                    return XPageResult.success(Collections.emptyList(), input, 0);
                }
                record.setOuIds(viewList.stream()
                        .map(SysOrganizationView::getId)
                        .collect(Collectors.toList()));
            }
        }
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserView> list = mapper.selectSysUserListByOuId(record);
        PageInfo<SysUserView> pageInfo = new PageInfo<>(list);
        return XPageResult.success(XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserViewOutput.class), input, pageInfo.getTotal());
    }
}
